//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// DESCRIPTION:
//	Console command registration, registration, and dispatching.
//  Also some base commands.
//
//-----------------------------------------------------------------------------

#ifndef __C_CMDS_H__
#define __C_CMDS_H__

#define ACTION_FORWARD				(1 << 1)
#define ACTION_BACKWARD				(1 << 2)
#define ACTION_TURNLEFT				(1 << 3)
#define ACTION_TURNRIGHT			(1 << 4)
#define ACTION_FIRE					(1 << 5)
#define ACTION_USE					(1 << 6)
#define ACTION_STRAFELEFT			(1 << 7)
#define ACTION_STRAFERIGHT			(1 << 8)
#define ACTION_SPEED				(1 << 9)
#define ACTION_STRAFE				(1 << 10)
#define ACTION_LOOKUP				(1 << 11)
#define ACTION_LOOKDOWN				(1 << 12)
#define ACTION_LOOKCENTER			(1 << 13)
#define ACTION_JUMP					(1 << 14)

typedef void (*CommandCallback)(int argc, char **argv);

typedef struct CommandFunction_s
{
    struct CommandFunction_s *next;
    char *name;
    CommandCallback func;
} CommandFunction;

void C_InitCmds();
void C_RegisterCommand(const char *name, CommandCallback func);
char *COM_Parse(char **data);
int C_TokeniseString(char *text, char ***argv, boolean expandMacros);
void C_ExecuteString(char *text);
void C_ExecuteMultiple(char *text);

CommandFunction *FindConsoleCommand(const char *name);

extern int ActionFlags;

#endif