//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// DESCRIPTION:
//	Console variable registration and functions.
//
//-----------------------------------------------------------------------------

#ifndef __C_CVAR_H__
#define __C_CVAR_H__

#include <stdio.h>
#include "doomtype.h"

#define	CVAR_ARCHIVE	1	// set to cause it to be saved to vars.rc
#define	CVAR_NOSET		2	// don't allow change from console at all,
                            // but can be set from the command line

typedef struct cvar_s {
    char *name;
    char *string;
    float value;
    int flags;
    boolean modified;

    struct cvar_s *next;
} cvar_t;

cvar_t *FindCvar(const char *name);
cvar_t *GetCvar(char *name, char *value, int flags);
cvar_t *SetCvar(char *name, char *value);
cvar_t *ForceSetCvar(char *name, char *value);
char *CvarString(char *name);

void C_InitCvars();
void C_SaveCvars(FILE *f);

void C_CvarSet(cvar_t *cv, char *value);
void C_CvarSetFloat(cvar_t *cv, float value);

#endif