//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// DESCRIPTION:
//	Initialises console variables.
//
//-----------------------------------------------------------------------------

#include <stdlib.h>
#include "c_cvar.h"
#include "d_englsh.h"

#define CVARINIT(name, value, flags) \
    &name, #name, value, flags,

extern cvar_t 
    *con_heightdiv,
    *con_staytime,
	*cl_run,
    *developer,
    *vid_showfps,
    *vid_width,
    *vid_height,
    *vid_vsync,
    *vid_fullscreen,
    *vid_uncapped,
    *screenblocks,
    *showmessages,
    *mouse_sensitivity,
    *snd_sfxvolume,
    *snd_musvolume,
    *snd_numchannels,
    *snd_randompitch,
    *snd_device,
    *snd_ignorepadding,
    *snd_drawoverlay;

extern cvar_t *chat_macros[10];

typedef struct {
    cvar_t **cvar;
    char *name;
    char *value;
    int flags;
} cvarinit_t;

static const cvarinit_t Inits[] = {
    CVARINIT(con_heightdiv,			"2",		CVAR_ARCHIVE)
    CVARINIT(con_staytime,          "3",        CVAR_ARCHIVE)
    CVARINIT(cl_run,				"0",        CVAR_ARCHIVE)
    CVARINIT(vid_showfps,			"0",		0)
    CVARINIT(vid_width,				"640",		CVAR_ARCHIVE)
    CVARINIT(vid_height,			"480",		CVAR_ARCHIVE)
    CVARINIT(vid_vsync,				"0",		CVAR_ARCHIVE)
    CVARINIT(vid_fullscreen,		"0",		CVAR_ARCHIVE)
    CVARINIT(vid_uncapped,			"1",		CVAR_ARCHIVE)
    CVARINIT(screenblocks,			"10",		CVAR_ARCHIVE)
    CVARINIT(showmessages,			"1",		CVAR_ARCHIVE)
    CVARINIT(mouse_sensitivity,		"5",		CVAR_ARCHIVE)
    CVARINIT(snd_sfxvolume,			"15",		CVAR_ARCHIVE)
    CVARINIT(snd_musvolume,			"15",		CVAR_ARCHIVE)
    CVARINIT(snd_numchannels,		"32",		CVAR_ARCHIVE)
    CVARINIT(developer,				"0",		0)
    CVARINIT(snd_randompitch,		"0",		CVAR_ARCHIVE)
    CVARINIT(snd_device,			"0",		CVAR_ARCHIVE)
    CVARINIT(snd_ignorepadding,		"1",		CVAR_ARCHIVE)
    CVARINIT(snd_drawoverlay,       "0",        0)

    &chat_macros[0],		"chatmacro0",		HUSTR_CHATMACRO0,		CVAR_ARCHIVE,
    &chat_macros[1],		"chatmacro1",		HUSTR_CHATMACRO1,		CVAR_ARCHIVE,
    &chat_macros[2],		"chatmacro2",		HUSTR_CHATMACRO2,		CVAR_ARCHIVE,
    &chat_macros[3],		"chatmacro3",		HUSTR_CHATMACRO3,		CVAR_ARCHIVE,
    &chat_macros[4],		"chatmacro4",		HUSTR_CHATMACRO4,		CVAR_ARCHIVE,
    &chat_macros[5],		"chatmacro5",		HUSTR_CHATMACRO5,		CVAR_ARCHIVE,
    &chat_macros[6],		"chatmacro6",		HUSTR_CHATMACRO6,		CVAR_ARCHIVE,
    &chat_macros[7],		"chatmacro7",		HUSTR_CHATMACRO7,		CVAR_ARCHIVE,
    &chat_macros[8],		"chatmacro8",		HUSTR_CHATMACRO8,		CVAR_ARCHIVE,
    &chat_macros[9],		"chatmacro9",		HUSTR_CHATMACRO9,		CVAR_ARCHIVE,

    NULL
};

void C_InitCvars()
{
    const cvarinit_t *i;

    for (i = Inits; i->cvar; i++)
    {
        *i->cvar = GetCvar(i->name, i->value, i->flags);
    }
}
