//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// DESCRIPTION:
//	Mission begin melt/wipe screen special effect.
//
//-----------------------------------------------------------------------------

#include "z_zone.h"
#include "i_video.h"
#include "multires.h"
#include "m_random.h"
#include "doomdef.h"
#include "doomstat.h"
#include "f_wipe.h"
#include "i_video.h"

//
//                       SCREEN WIPE PACKAGE
//

// when zero, stop the wipe
static boolean go = 0;

static screen_t wipe_scr_start;
static screen_t wipe_scr_end;
static screen_t *wipe_scr;

void wipe_shittyColMajorXform(short *array,
                              int width,
                              int height)
{
    int x;
    int y;
    short *dest;

    dest = (short *)Z_Malloc(width * height * 2, PU_STATIC, 0);

    for (y = 0; y < height; y++)
        for (x = 0; x < width; x++)
            dest[x * height + y] = array[y * width + x];

    memcpy(array, dest, width * height * 2);

    Z_Free(dest);
}

int wipe_initColorXForm(int width,
                        int height,
                        int ticks)
{
    memcpy(wipe_scr->data, wipe_scr_start.data, width * height * 1);
    return 0;
}

int wipe_doColorXForm(int width,
                      int height,
                      int ticks)
{
    boolean changed;
    byte *w;
    byte *e;
    int newval;

    changed = false;
    w = wipe_scr->data;
    e = wipe_scr_end.data;

    while (w != wipe_scr->data + width * height)
    {
        if (*w != *e)
        {
            if (*w > *e)
            {
                newval = *w - ticks;
                if (newval < *e)
                    *w = *e;
                else
                    *w = newval;
                changed = true;
            }
            else if (*w < *e)
            {
                newval = *w + ticks;
                if (newval > *e)
                    *w = *e;
                else
                    *w = newval;
                changed = true;
            }
        }
        w++;
        e++;
    }

    return !changed;
}

int wipe_exitColorXForm(int width,
                        int height,
                        int ticks)
{
    return 0;
}

static int *y;

int wipe_initMelt(int width,
                  int height,
                  int ticks)
{
    int i, r;

    I_LockScreen(wipe_scr);
    I_LockScreen(&wipe_scr_start);
    I_LockScreen(&wipe_scr_end);

    // copy start screen to main screen
    memcpy(wipe_scr->data, wipe_scr_start.data, width * height);

    // makes this wipe faster (in theory)
    // to have stuff in column-major format
    wipe_shittyColMajorXform((short *)wipe_scr_start.data, width / 2, height);
    wipe_shittyColMajorXform((short *)wipe_scr_end.data, width / 2, height);

    // setup initial column positions
    // (y<0 => not ready to scroll yet)
    y = (int *)Z_Malloc(width * sizeof(int), PU_STATIC, 0);
    y[0] = -(M_Random() % 16);
    for (i = 1; i < width; i++)
    {
        r = (M_Random() % 3) - 1;
        y[i] = y[i - 1] + r;
        if (y[i] > 0)
            y[i] = 0;
        else if (y[i] == -16)
            y[i] = -15;
    }

    I_UnlockScreen(wipe_scr);
    I_UnlockScreen(&wipe_scr_start);
    I_UnlockScreen(&wipe_scr_end);

    return 0;
}

int wipe_doMelt(int width,
                int height,
                int ticks)
{
    int i;
    int j;
    int dy;
    int idx;

    short *s;
    short *d;
    boolean done = true;

    width = width / 2;

    I_LockScreen(&wipe_scr_end);
    I_LockScreen(&wipe_scr_start);
    I_LockScreen(wipe_scr);

    while (ticks--)
    {
        for (i = 0; i < width; i++)
        {
            if (y[i] < 0)
            {
                y[i]++;
                done = false;
            }
            else if (y[i] < height)
            {
                dy = (y[i] < 16) ? y[i] + 1 : 8;
                if (y[i] + dy >= height)
                    dy = height - y[i];
                s = &((short *)wipe_scr_end.data)[i * height + y[i]];
                d = &((short *)wipe_scr->data)[y[i] * width + i];
                idx = 0;
                for (j = dy; j; j--)
                {
                    d[idx] = *(s++);
                    idx += width;
                }
                y[i] += dy;
                s = &((short *)wipe_scr_start.data)[i * height];
                d = &((short *)wipe_scr->data)[y[i] * width + i];
                idx = 0;
                for (j = height - y[i]; j; j--)
                {
                    d[idx] = *(s++);
                    idx += width;
                }
                done = false;
            }
        }
    }

    I_UnlockScreen(&wipe_scr_end);
    I_UnlockScreen(&wipe_scr_start);
    I_UnlockScreen(wipe_scr);

    return done;
}

int wipe_exitMelt(int width,
                  int height,
                  int ticks)
{
    Z_Free(y);
    I_FreeScreen(&wipe_scr_start);
    I_FreeScreen(&wipe_scr_end);
    return 0;
}

int wipe_StartScreen(int x,
                     int y,
                     int width,
                     int height)
{
    I_AllocScreen(&wipe_scr_start, width, height, 1);
    I_LockScreen(&wipe_scr_start);
    I_ReadScreen(wipe_scr_start.data);
    I_UnlockScreen(&wipe_scr_start);
    return 0;
}

int wipe_EndScreen(int x,
                   int y,
                   int width,
                   int height)
{
    I_AllocScreen(&wipe_scr_end, width, height, 1);
    I_LockScreen(&wipe_scr_end);
    I_ReadScreen(wipe_scr_end.data);
    I_UnlockScreen(&wipe_scr_end);
    I_LockScreen(&wipe_scr_start);
    V_DrawBlock(x, y, &screens[0], width, height, wipe_scr_start.data); // restore start scr.
    I_UnlockScreen(&wipe_scr_start);
    return 0;
}

boolean wipe_ScreenWipe(int wipeno,
                    int x,
                    int y,
                    int width,
                    int height,
                    int ticks)
{
    int rc;
    static int (*wipes[])(int, int, int) =
        {
            wipe_initColorXForm, wipe_doColorXForm, wipe_exitColorXForm,
            wipe_initMelt, wipe_doMelt, wipe_exitMelt};

    // initial stuff
    if (!go)
    {
        go = true;
        // wipe_scr = (byte *) Z_Malloc(width*height, PU_STATIC, 0); // DEBUG
        wipe_scr = &screens[0];
        (*wipes[wipeno * 3])(width, height, ticks);
    }

    // do a piece of wipe-in
    V_MarkRect(0, 0, width, height);
    rc = (*wipes[wipeno * 3 + 1])(width, height, ticks);
    //  V_DrawBlock(x, y, 0, width, height, wipe_scr); // DEBUG

    // final stuff
    if (rc)
    {
        go = false;
        (*wipes[wipeno * 3 + 2])(width, height, ticks);
    }

    return !go;
}
