//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// DESCRIPTION:  none
//
//-----------------------------------------------------------------------------

#ifndef __HULIB__
#define __HULIB__

// We are referring to patches.
#include "r_defs.h"

// background and foreground screen numbers
// different from other modules.
#define FG (&screens[0])

// font stuff
#define HU_CHARERASE KEY_BACKSPACE

#define HU_MAXLINES 4
#define HU_MAXLINELENGTH 80

//
// Typedefs of widgets
//

// Text Line widget
//  (parent of Scrolling Text and Input Text widgets)
typedef struct
{
    // left-justified position of scrolling text window
    int x;
    int y;

    patch_t **f;                  // font
    int sc;                       // start character
    char l[HU_MAXLINELENGTH + 1]; // line of text
    int len;                      // current line length

    // whether this line needs to be udpated
    int needsupdate;

} hu_textline_t;

// Scrolling Text window widget
//  (child of Text Line widget)
typedef struct
{
    hu_textline_t l[HU_MAXLINES]; // text lines to draw
    int h;                        // height in lines
    int cl;                       // current line number

    // pointer to boolean stating whether to update window
    boolean *on;
    boolean laston; // last value of *->on.

} hu_stext_t;

// Input Text Line widget
//  (child of Text Line widget)
typedef struct
{
    hu_textline_t l; // text line to input on

    // left margin past which I am not to delete characters
    int lm;

    // pointer to boolean stating whether to update window
    boolean *on;
    boolean laston; // last value of *->on;

} hu_itext_t;

//
// Widget creation, access, and update routines
//

// initializes heads-up widget library
void HUlib_init(void);

//
// textline code
//

// clear a line of text
void HUlib_clearTextLine(hu_textline_t *t);

void HUlib_initTextLine(hu_textline_t *t, int x, int y, patch_t **f, int sc);

// returns success
boolean HUlib_addCharToTextLine(hu_textline_t *t, char ch);

// returns success
boolean HUlib_delCharFromTextLine(hu_textline_t *t);

// draws tline
void HUlib_drawTextLine(hu_textline_t *l, boolean drawcursor);

// erases text line
void HUlib_eraseTextLine(hu_textline_t *l);

//
// Scrolling Text window widget routines
//

// ?
void HUlib_initSText(hu_stext_t *s,
                     int x,
                     int y,
                     int h,
                     patch_t **font,
                     int startchar,
                     boolean *on);

// add a new line
void HUlib_addLineToSText(hu_stext_t *s);

// ?
void HUlib_addMessageToSText(hu_stext_t *s,
                             char *prefix,
                             char *msg);

// draws stext
void HUlib_drawSText(hu_stext_t *s);

// erases all stext lines
void HUlib_eraseSText(hu_stext_t *s);

// Input Text Line widget routines
void HUlib_initIText(hu_itext_t *it,
                     int x,
                     int y,
                     patch_t **font,
                     int startchar,
                     boolean *on);

// enforces left margin
void HUlib_delCharFromIText(hu_itext_t *it);

// enforces left margin
void HUlib_eraseLineFromIText(hu_itext_t *it);

// resets line and left margin
void HUlib_resetIText(hu_itext_t *it);

// left of left-margin
void HUlib_addPrefixToIText(hu_itext_t *it,
                            char *str);

// whether eaten
boolean
HUlib_keyInIText(hu_itext_t *it,
                 unsigned char ch);

void HUlib_drawIText(hu_itext_t *it);

// erases all itext lines
void HUlib_eraseIText(hu_itext_t *it);

#endif