//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// DESCRIPTION:
//	Memory allocation wrappers
//
//-----------------------------------------------------------------------------

#ifndef __M_ALLOC_H__
#define __M_ALLOC_H__

#include <ctype.h>

void *_M_Malloc(size_t size, const char *file, int line, const char *function);
void *_M_Calloc(size_t num, size_t size, const char *file, int line, const char *function);
void *_M_Realloc(void *block, size_t newsize, const char *file, int line, const char *function);
void _M_Free(void *block, const char *file, int line, const char *function);

#define M_Malloc(size) _M_Malloc(size, __FILE__, __LINE__, __FUNCTION__)
#define M_Calloc(num, size) _M_Calloc(num, size, __FILE__, __LINE__, __FUNCTION__)
#define M_Realloc(block, newsize) _M_Realloc(block, newsize, __FILE__, __LINE__, __FUNCTION__)
#define M_Free(block) _M_Free(block, __FILE__, __LINE__, __FUNCTION__)

#endif