//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// DESCRIPTION:
//	Fixed point implementation.
//
//-----------------------------------------------------------------------------

#include "stdlib.h"

#include "doomtype.h"
#include "i_system.h"

#include "m_fixed.h"

/* [Petteri] */
#include <assert.h>

// Fixme. __USE_C_FIXED__ or something.

fixed_t
FixedMul(fixed_t a,
         fixed_t b)
{
#if defined(__WATCOMC__) || defined(_MSC_VER)
    /* [Petteri] FIXME: inline assembler #pragma would be a better idea: */
    double da = ((double)a) * (1.0 / ((double)FRACUNIT));
    double db = ((double)b) * (1.0 / ((double)FRACUNIT));
    return (fixed_t)(da * db * ((double)FRACUNIT));
#else
    return ((long long)a * (long long)b) >> FRACBITS;
#endif
}

//
// FixedDiv, C version.
//

fixed_t
FixedDiv(fixed_t a,
         fixed_t b)
{
    if ((abs(a) >> 14) >= abs(b))
        return (a ^ b) < 0 ? MININT : MAXINT;
    return FixedDiv2(a, b);
}

fixed_t
FixedDiv2(fixed_t a,
          fixed_t b)
{
#if 1
    long long c;
    c = ((long long)a<<16) / ((long long)b);
    return (fixed_t) c;
#else

    double c;

    c = ((double)a) / ((double)b) * FRACUNIT;

    assert(c < 2147483648.0 && c >= -2147483648.0);

    if (c >= 2147483648.0 || c < -2147483648.0)
        I_Error("FixedDiv: divide by zero");
    return (fixed_t)c;
#endif
}
