//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------

#ifndef __M_STREAM_H__
#define __M_STREAM_H__

#include <stddef.h>
#include "doomtype.h"

typedef struct
{
    byte *data;
    byte *dataEnd;
    byte *orig;
    boolean dynamic; // whether the buffer can be realloced when low on space
    boolean managed; // whether the buffer was created by Stream_Alloc
} stream_t;

void Stream_Alloc(stream_t *stream, size_t length, boolean dynamic);
void Stream_InitSize(stream_t *stream, byte *buffer, size_t size, boolean dynamic);
void Stream_InitPtr(stream_t *stream, byte *begin, byte *end, boolean dynamic);
void Stream_Free(stream_t *stream); // Only neccesary if you used Stream_Alloc, but doesn't hurt to call anyway

size_t Stream_GetSize(stream_t *stream);
size_t Stream_GetPosition(stream_t *stream);
size_t Stream_GetAvailable(stream_t *stream);

boolean Stream_Write(stream_t *stream, byte *source, size_t length);
boolean Stream_WriteByte(stream_t *stream, byte value);
boolean Stream_WriteShort(stream_t *stream, short value);
boolean Stream_WriteInt(stream_t *stream, int value);
boolean Stream_WriteString(stream_t *stream, const char *value);

boolean Stream_Read(stream_t *stream, byte *dest, size_t length);
byte Stream_ReadByte(stream_t *stream);
short Stream_ReadShort(stream_t *stream);
int Stream_ReadInt(stream_t *stream);
const char *Stream_ReadString(stream_t *stream);

#endif