//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------
#include <stdlib.h>
#include <stdio.h>
#include "doomdef.h"
#include "doomstat.h"
#include "m_argv.h"

#include "d_event.h"
#include "r_plane.h"
#include "r_state.h"
#include "r_things.h"
#include "am_map.h"

#include "multires.h"

#include "v_video.h"
#include "r_draw.h"
#include "c_cvar.h"
#include "m_alloc.h"

///////////////////////////////////////
//v_video.c stuff
///////////////////////////////////////

screen_t screens[2];

extern cvar_t *vid_width;
extern cvar_t *vid_height;

///////////////////////////////////////
//r_draw.c stuff
///////////////////////////////////////

//
// R_DrawColumn
// Source is the top of the column to scale.
//
lighttable_t *dc_colormap;
int dc_x;
int dc_yl;
int dc_yh;
fixed_t dc_iscale;
fixed_t dc_texturemid;

// first pixel in a column (possibly virtual)
byte *dc_source;

byte *dc_translation;
byte *translationtables;

//r_drawspan:
int ds_y;
int ds_x1;
int ds_x2;

lighttable_t *ds_colormap;

fixed_t ds_xfrac;
fixed_t ds_yfrac;
fixed_t ds_xstep;
fixed_t ds_ystep;

// start of a 64*64 tile image
byte *ds_source;

// just for profiling
int dscount;

byte *ylookup[MAXHEIGHT];
int columnofs[MAXWIDTH];

#define FUZZTABLE 50
#define FUZZOFF 1
int fuzzoffset[FUZZTABLE] =
    {
        FUZZOFF, -FUZZOFF, FUZZOFF, -FUZZOFF, FUZZOFF, FUZZOFF, -FUZZOFF,
        FUZZOFF, FUZZOFF, -FUZZOFF, FUZZOFF, FUZZOFF, FUZZOFF, -FUZZOFF,
        FUZZOFF, FUZZOFF, FUZZOFF, -FUZZOFF, -FUZZOFF, -FUZZOFF, -FUZZOFF,
        FUZZOFF, -FUZZOFF, -FUZZOFF, FUZZOFF, FUZZOFF, FUZZOFF, FUZZOFF, -FUZZOFF,
        FUZZOFF, -FUZZOFF, FUZZOFF, FUZZOFF, -FUZZOFF, -FUZZOFF, FUZZOFF,
        FUZZOFF, -FUZZOFF, -FUZZOFF, -FUZZOFF, -FUZZOFF, FUZZOFF, FUZZOFF,
        FUZZOFF, FUZZOFF, -FUZZOFF, FUZZOFF, FUZZOFF, -FUZZOFF, FUZZOFF};

///////////////////////////////////////
//Now, Finally, comes my stuff
///////////////////////////////////////

void multires_setres(void)
{
    int p;

    p = M_CheckParm("-width");
    if (p && p < myargc - 1)
        SCREENWIDTH = atoi(myargv[p + 1]);
    else
        SCREENWIDTH = (int)vid_width->value;

    p = M_CheckParm("-height");
    if (p && p < myargc - 1)
        SCREENHEIGHT = atoi(myargv[p + 1]);
    else
        SCREENHEIGHT = (int)vid_height->value;

    printf("Resolution: %d x %d\n", SCREENWIDTH, SCREENHEIGHT);

    if ((SCREENHEIGHT * 8) == (SCREENWIDTH * 5))
        weirdaspect = 1;
    else if ((SCREENHEIGHT * 4) == (SCREENWIDTH * 3))
        weirdaspect = 0;
    else
    {
        weirdaspect = 0;
        printf("Warning: Bad aspect ratio, things might look screwy\n");
    }

    //allocate all the res-dependant vars
    resinit_r_plane_c();
    resinit_am_map_c();
    resinit_r_draw_c();
    xtoviewangle = (angle_t *)M_Calloc(SCREENWIDTH + 1, sizeof(angle_t));
    negonearray = (short *)M_Calloc(SCREENWIDTH, sizeof(short));
    screenheightarray = (short *)M_Calloc(SCREENWIDTH, sizeof(short));
}