//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------

#ifndef __V_VIDEO__
#define __V_VIDEO__

#include "doomtype.h"
#include "doomdef.h"
// Needed because we are refering to patches.
#include "r_data.h"

typedef struct {
    void *platdata;
    int width;
    int height;
    int depth;		// in bytes
    int pitch;
    byte *data;
    int locked;		// number of times this was locked
} screen_t;

// Column drawer pointers
typedef void (*ColumnDrawer)(byte *source, byte *dest, int pitch, int count);
typedef void (*ColumnDrawerStretched)(byte *source, byte *dest, int pitch, int count, int yinc);

// Types of column drawers
enum
{
    D_DrawPatch,			// basic drawer
    D_DrawPatchTranslated,	// translated via V_TranslationTable
    D_NumColumnDrawers
};

void V_BaseDraw(int type, int x, int y, screen_t *scrn, patch_t *patch);
void V_BaseDrawStretched(int type, int x, int y, screen_t *scrn, patch_t *patch, int width, int height);
void V_BaseDrawIndirect(int type, int x, int y, screen_t *scrn, patch_t *patch);

void V_DrawString(int x, int y, int cr, const char *str);

void V_PalleteIndexToColor(byte index, int redness, byte *r, byte *g, byte *b);
byte V_ClosestColor(byte r, byte g, byte b);

void V_CalculateGamma();

// Allocates buffer screens, call before R_Init.
void V_Init(void);

void V_CopyRect(int srcx, int srcy, screen_t *srcscrn, int width, int height, int destx, int desty, screen_t *destscrn);

#define V_DrawPatch(x, y, scrn, patch) \
    V_BaseDraw(D_DrawPatch, x, y, scrn, patch)
#define V_DrawPatchDirect(x, y, scrn, patch) \
    V_BaseDraw(D_DrawPatch, x, y, scrn, patch)
#define V_DrawStretchedPatch(x, y, scrn, patch, width, height) \
    V_BaseDrawStretched(D_DrawPatch, x, y, scrn, patch, width, height)
#define V_DrawPatchInDirect(x, y, scrn, patch) \
    V_BaseDrawIndirect(D_DrawPatch, x, y, scrn, patch)
#define V_DrawPatchTranslated(x, y, scrn, patch) \
    V_BaseDraw(D_DrawPatchTranslated, x, y, scrn, patch)
#define V_DrawStretchedTranslatedPatch(x, y, scrn, patch, width, height) \
    V_BaseDrawStretched(D_DrawPatchTranslated, x, y, scrn, patch, width, height)
#define V_DrawTranslatedPatchInDirect(x, y, scrn, patch) \
    V_BaseDrawIndirect(D_DrawPatchTranslated, x, y, scrn, patch)

// Draw a linear block of pixels into the view buffer.
void V_DrawBlock(int x, int y, screen_t *scrn, int width, int height, byte *src);

// Reads a linear block of pixels into the view buffer.
void V_GetBlock(int x, int y, screen_t *scrn, int width, int height, byte *dest);

void V_MarkRect(int x, int y, int width, int height);
void V_ClearScreen(screen_t *scrn);

// Translation stuff
extern byte *V_TranslationTables;
extern byte *V_TranslationTable;

// symbolic indices into color translation table pointer array
enum
{
    CR_BRICK,
    CR_TAN,
    CR_GRAY,
    CR_GREEN,
    CR_BROWN,
    CR_GOLD,
    CR_RED,
    CR_BLUE,
    CR_BLUE2,
    CR_ORANGE,
    CR_YELLOW,
    CR_BLACK,
    CR_LIMIT
};

#endif