//-----------------------------------------------------------------------------
//
// Copyright 2019-2020 Sean Baggaley
// Copyright 1993-1997 id Software
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of the copyright holder nor the names of its 
//    contributors may be used to endorse or promote products derived from this
//    software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <assert.h>
#include "doomstat.h"
#include "i_system.h"
#include "i_video.h"
#include "multires.h"
#include "m_argv.h"
#include "d_main.h"
#include "doomdef.h"
#include "version.h"
#include "i_win32.h"
#include "resource.h"
#include "c_cvar.h"
#include "hginfo.h"

static int closed = 0;

static int disableVerticalMouse = 0;

static int windowActive = 0;
boolean grabMouse;
static boolean grabTyping = false;

HWND win;
static HINSTANCE inst;

cvar_t *vid_width;
cvar_t *vid_height;
cvar_t *vid_fullscreen;

void WinError(char *msg)
{
    printf("Windows Error: %s, GetLastError(): %u\n", msg, GetLastError());
    exit(EXIT_FAILURE);
}


/*static int	lastmousex = 0;
static int	lastmousey = 0;
boolean		mousemoved = false;
boolean		shmFinished;*/

void I_GetEvent(void)
{
    MSG msg;
    POINT point;
    static LONG prevX, prevY;
    static int hadMouse = 0;
    event_t event;
    RECT rect;
    int lb, rb;
    static int prevlb = 0, prevrb = 0;

    /* Dispatch all messages: */
    while (PeekMessage(&msg, NULL, 0, 0xFFFFFFFF, PM_REMOVE))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    /* Check mouse and generate events if necessary: */
    if (!GetCursorPos(&point))
        WinError("GetCursorPos() failed");
    if (hadMouse && windowActive)
    {
        lb = (GetAsyncKeyState(VK_LBUTTON) < 0);
        rb = (GetAsyncKeyState(VK_RBUTTON) < 0);

        if ((prevX != point.x) || (prevY != point.y) ||
            (prevlb != lb) || (prevrb != rb))
        {
            event.type = ev_mouse;
            event.data1 = lb | (rb << 1);
            event.data2 = (point.x - prevX) << 2;
            if (disableVerticalMouse)
                event.data3 = 0;
            else
                event.data3 = (prevY - point.y) << 2;
            prevX = point.x;
            prevY = point.y;
            prevlb = lb;
            prevrb = rb;
            //D_PostEvent(&event);
        }

        if (grabMouse)
        {
            GetWindowRect(win, &rect);
            if (!SetCursorPos((rect.left + rect.right) / 2,
                              (rect.top + rect.bottom) / 2))
                WinError("SetCursorPos() failed");
            prevX = (rect.left + rect.right) / 2;
            prevY = (rect.top + rect.bottom) / 2;
        }
    }
    else
    {
        prevX = point.x;
        prevY = point.y;
        hadMouse = 1;
    }
}

//
// I_StartTic
//
void I_StartTic(void)
{
    I_GetEvent();
}

/* Stuff for multiresolution support: */
int SCREENWIDTH;
int SCREENHEIGHT;
int weirdaspect;
byte *hicolortable;
short hicolortransmask1, hicolortransmask2;

#define POINTER_WARP_COUNTDOWN 1

int X_width;
int X_height;

/* Multiplication - we'll let windows stretch instead */
static int multiply = 1;

int TranslateKey(unsigned k)
{
    /*wtf?    if ( (k >= VK_0) && (k <= VK_9) )*/
    if ((k >= 0x30) && (k <= 0x39))
        return (k - 0x30 + '0');
    if ((k >= 0x41) && (k <= 0x5a))
        return (k - 0x41 + 'a');

#define K(a, b) \
    case a:     \
        return b;
    switch (k)
    {
        K(VK_LEFT, KEY_LEFTARROW);
        K(VK_RIGHT, KEY_RIGHTARROW);
        K(VK_UP, KEY_UPARROW);
        K(VK_DOWN, KEY_DOWNARROW);
        K(VK_BACK, KEY_BACKSPACE);
        K(VK_TAB, KEY_TAB);
        K(VK_RETURN, KEY_ENTER);
        K(VK_SHIFT, KEY_RSHIFT);
        K(VK_CONTROL, KEY_RCTRL);
        K(VK_MENU, KEY_RALT);
        K(VK_PAUSE, KEY_PAUSE);
        K(VK_ESCAPE, KEY_ESCAPE);
        K(VK_SPACE, ' ');
        K(VK_DELETE, KEY_BACKSPACE);
        K(VK_ADD, '+');
        K(VK_SUBTRACT, KEY_MINUS);
        K(0xBC, ',');
        K(0xBE, '.');
        K(VK_F1, KEY_F1);
        K(VK_F2, KEY_F2);
        K(VK_F3, KEY_F3);
        K(VK_F4, KEY_F4);
        K(VK_F5, KEY_F5);
        K(VK_F6, KEY_F6);
        K(VK_F7, KEY_F7);
        K(VK_F8, KEY_F8);
        K(VK_F9, KEY_F9);
        K(VK_F10, KEY_F10);
        K(VK_F11, KEY_F11);
        K(VK_F12, KEY_F12);
        K(VK_OEM_2, KEY_GRAVE);
        K(VK_OEM_1, KEY_SEMICOLON);
        K(VK_PRIOR, KEY_PGUP);
        K(VK_NEXT, KEY_PGDN);
    }

    return 0;
}

static LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wparam, LPARAM lparam)
{
    event_t event;
    RECT rect;
    UINT scancode;

    switch (message)
    {
    case WM_DESTROY:
        if (grabMouse)
        {
            ClipCursor(NULL);
            ShowCursor(TRUE);
        }
        fprintf(stderr, "WM_DESTROY\n");
        PostQuitMessage(0);
        I_Quit();
        closed = 1;
        break;

    case WM_MOVE:
        GetWindowRect(win, &rect);
        fprintf(stderr, "%u,%u - %u, %u\n",
                rect.left, rect.top, rect.right, rect.bottom);
        //ClipCursor(&rect);
        break;

    case WM_ACTIVATE:
        if (LOWORD(wparam))
        {
            if (!windowActive)
            {
                if (grabMouse)
                {
                    ClipCursor(NULL); /* helps with Win95? */
                    GetWindowRect(win, &rect);
                    fprintf(stderr, "%u,%u - %u, %u\n",
                            rect.left, rect.top, rect.right, rect.bottom);
                    ShowCursor(FALSE);
                }
            }
            windowActive = 1;
        }
        else
        {
            if (grabMouse)
            {
                ClipCursor(NULL);
                ShowCursor(TRUE);
            }
            windowActive = 0;
        }
        return DefWindowProc(hwnd, message, wparam, lparam);

    case WM_KEYDOWN:
    case WM_KEYUP:
        event.type = (message == WM_KEYDOWN ? ev_keydown : ev_keyup);
        event.data1 = TranslateKey(wparam);

        scancode = (lparam >> 16) & 0x00FF;

        event.data2 = wparam;
        event.data3 = scancode;

        D_PostEvent(&event);

        break;

    case WM_CHAR:
    case WM_SYSCHAR:
        if (!grabTyping)
            break;

        event.type = ev_type;
        event.data1 = wparam;

        D_PostEvent(&event);

        break;

    default:
        return (DefWindowProc(hwnd, message, wparam, lparam));
    }

    return 0;
}

void I_StartTyping()
{
    grabTyping = true;
}

void I_StopTyping()
{
    grabTyping = false;
}

void I_ShutdownGraphics(void)
{
    I_ShutdownD3D9();
    if (grabMouse)
    {
        ClipCursor(NULL);
        ShowCursor(TRUE);
    }
}

//
// I_StartFrame
//
void I_StartFrame(void)
{
    // er?
}

//
// I_UpdateNoBlit
//
void I_UpdateNoBlit(void)
{
    // what is this?
}

void I_InitGraphics(void)
{
    static int firsttime = 1;

    WNDCLASS wc;
    int frameX, frameY, capY;
    RECT rect;
    int width, height;
    int xPos, yPos;
    int flags;

    if (!firsttime)
        return;
    firsttime = 0;

    if (M_CheckParm("-2"))
        multiply = 2;

    if (M_CheckParm("-3"))
        multiply = 3;

    if (M_CheckParm("-4"))
        multiply = 4;

    X_width = SCREENWIDTH * multiply;
    X_height = SCREENHEIGHT * multiply;

    // check if the user wants to grab the mouse (quite unnice)
    grabMouse = !!M_CheckParm("-grabmouse");

    /* [Petteri] New: Option to disable mouse vertical movement - useful
       for players used to Quake: */
    disableVerticalMouse = !!M_CheckParm("-novertmouse");

    /* Build and initialize the window: */

    inst = (HINSTANCE)GetModuleHandle(NULL);

    frameX = GetSystemMetrics(SM_CXFRAME);
    frameY = GetSystemMetrics(SM_CYFRAME);
    capY = GetSystemMetrics(SM_CYCAPTION);

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = inst;
    wc.hIcon = LoadIcon(inst, MAKEINTRESOURCE(IDI_ICON1));
    if (grabMouse)
        wc.hCursor = LoadCursor(0, IDC_ARROW);
    else
        wc.hCursor = LoadCursor(0, IDC_ARROW);
    /*wc.hbrBackground = GetStockObject( WHITE_BRUSH );*/
    wc.hbrBackground = NULL;
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "DoomWindowClass";

    RegisterClass(&wc);

    width = X_width + 2 * frameX;
    height = X_height + 2 * frameY + capY;

    flags = WS_OVERLAPPEDWINDOW | WS_VISIBLE;
    if (vid_fullscreen->value)
    {
        flags |= WS_EX_TOPMOST | WS_POPUP;
    }

    win = CreateWindow("DoomWindowClass", PACKAGE_NAME,
                       flags, 0, 0, width, height,
                       NULL, NULL, inst, NULL);

    /* Display the window: */
    ShowWindow(win, SW_SHOW);
    UpdateWindow(win);

    I_UpdateWindowTitle("");

    GetClientRect(win, &rect);
    fprintf(stderr, "I_InitGraphics: Client area: %ux%u\n",
            rect.right - rect.left, rect.bottom - rect.top);

    if ((rect.right - rect.left) != X_width)
    {
        fprintf(stderr, "I_InitGraphics: Fixing width\n");
        width += X_width - (rect.right - rect.left);
        MoveWindow(win, 0, 0, width, height, TRUE);
    }
    if ((rect.bottom - rect.top) != X_height)
    {
        fprintf(stderr, "I_InitGraphics: Fixing height\n");
        height += X_height - (rect.bottom - rect.top);
        MoveWindow(win, 0, 0, width, height, TRUE);
    }

    GetClientRect(win, &rect);
    fprintf(stderr, "I_InitGraphics: Client area: %ux%u\n",
            rect.right - rect.left, rect.bottom - rect.top);

    GetWindowRect(win, &rect);

    xPos = (GetSystemMetrics(SM_CXSCREEN) - rect.right)/2;
    yPos = (GetSystemMetrics(SM_CYSCREEN) - rect.bottom)/2;

    SetWindowPos(win, 0, xPos, yPos, 0, 0, SWP_NOZORDER|SWP_NOSIZE);
    GetClientRect(win, &rect);


    I_InitD3D9();
}

void I_UpdateWindowTitle(const char *fmt, ...)
{
    va_list args;
    char buf[256];
    char title[512];
    
    va_start (args, fmt);
    vsprintf (buf, fmt, args);
    va_end (args);

    if (buf[0] != '\0')
        sprintf(title, "%s -- %s v%s (%s) r%s", buf, PACKAGE_NAME, PACKAGE_VERSION, __DATE__, HG_REVISION_STR);
    else
        sprintf(title, "%s v%s (%s) r%s", PACKAGE_NAME, PACKAGE_VERSION, __DATE__, HG_REVISION_STR);

    SetWindowText(win, title);
}